@@ -1,22 +1,29 @@
+use std::{error, fmt, io};
+
 use derive_more::{Display, From};
 use ntex::util::Either;
-use std::{error, io};
 
 /// Errors which can occur when attempting to handle mqtt connection.
-#[derive(Debug)]
+#[derive(Debug, Display)]
 pub enum MqttError<E> {
     /// Publish handler service error
+    #[display(fmt = "Service error")]
     Service(E),
     /// Protocol error
+    #[display(fmt = "Mqtt protocol error: {}", _0)]
     Protocol(ProtocolError),
     /// Handshake timeout
     HandshakeTimeout,
     /// Peer disconnect
-    Disconnected,
+    #[display(fmt = "Peer is disconnected, error: {:?}", _0)]
+    Disconnected(Option<io::Error>),
     /// Server error
+    #[display(fmt = "Server error: {}", _0)]
     ServerError(&'static str),
 }
 
+impl<E: fmt::Debug> error::Error for MqttError<E> {}
+
 /// Protocol level errors
 #[derive(Debug, Display, From)]
 pub enum ProtocolError {
@@ -44,9 +51,6 @@ pub enum ProtocolError {
     /// Keep alive timeout
     #[display(fmt = "Keep alive timeout")]
     KeepAliveTimeout,
-    /// Unexpected io error
-    #[display(fmt = "Unexpected io error: {}", _0)]
-    Io(io::Error),
 }
 
 impl error::Error for ProtocolError {}
@@ -57,11 +61,23 @@ impl<E> From<ProtocolError> for MqttError<E> {
     }
 }
 
+impl<E> From<io::Error> for MqttError<E> {
+    fn from(err: io::Error) -> Self {
+        MqttError::Disconnected(Some(err))
+    }
+}
+
+impl<E> From<Either<io::Error, io::Error>> for MqttError<E> {
+    fn from(err: Either<io::Error, io::Error>) -> Self {
+        MqttError::Disconnected(Some(err.into_inner()))
+    }
+}
+
 impl<E> From<Either<DecodeError, io::Error>> for MqttError<E> {
     fn from(err: Either<DecodeError, io::Error>) -> Self {
         match err {
             Either::Left(err) => MqttError::Protocol(ProtocolError::Decode(err)),
-            Either::Right(err) => MqttError::Protocol(ProtocolError::Io(err)),
+            Either::Right(err) => MqttError::Disconnected(Some(err)),
         }
     }
 }
@@ -70,16 +86,7 @@ impl<E> From<Either<EncodeError, io::Error>> for MqttError<E> {
     fn from(err: Either<EncodeError, io::Error>) -> Self {
         match err {
             Either::Left(err) => MqttError::Protocol(ProtocolError::Encode(err)),
-            Either::Right(err) => MqttError::Protocol(ProtocolError::Io(err)),
-        }
-    }
-}
-
-impl From<Either<DecodeError, io::Error>> for ProtocolError {
-    fn from(err: Either<DecodeError, io::Error>) -> Self {
-        match err {
-            Either::Left(err) => ProtocolError::Decode(err),
-            Either::Right(err) => ProtocolError::Io(err),
+            Either::Right(err) => MqttError::Disconnected(Some(err)),
         }
     }
 }
