@@ -1,12 +1,14 @@
 //! Framed transport dispatcher
 use std::task::{Context, Poll};
-use std::{cell::RefCell, collections::VecDeque, future::Future, pin::Pin, rc::Rc, time};
+use std::{
+    cell::Cell, cell::RefCell, collections::VecDeque, future::Future, pin::Pin, rc::Rc, time,
+};
 
-pub(crate) use ntex::framed::{DispatchItem, ReadTask, State, Timer, Write, WriteTask};
-
-use ntex::codec::{AsyncRead, AsyncWrite, Decoder, Encoder};
+use ntex::codec::{Decoder, Encoder};
+use ntex::io::{DispatchItem, IoBoxed, IoRef, IoStatusUpdate, RecvError};
 use ntex::service::{IntoService, Service};
-use ntex::{time::Seconds, util::Either, util::Pool};
+use ntex::time::Seconds;
+use ntex::util::{ready, Pool};
 
 type Response<U> = <U as Encoder>::Item;
 
@@ -14,29 +16,39 @@ pin_project_lite::pin_project! {
     /// Dispatcher for mqtt protocol
     pub(crate) struct Dispatcher<S, U>
     where
-        S: Service<Request = DispatchItem<U>, Response = Option<Response<U>>>,
+        S: Service<DispatchItem<U>, Response = Option<Response<U>>>,
         S::Error: 'static,
         S::Future: 'static,
         U: Encoder,
         U: Decoder,
        <U as Encoder>::Item: 'static,
     {
-        service: S,
         codec: U,
-        state: State,
-        inner: Rc<RefCell<DispatcherState<S, U>>>,
+        service: S,
+        state: Rc<RefCell<DispatcherState<S, U>>>,
+        inner: DispatcherInner,
         st: IoDispatcherState,
+        flags: Cell<Flags>,
         pool: Pool,
-        timer: Timer,
-        updated: time::Instant,
-        keepalive_timeout: Seconds,
         #[pin]
         response: Option<S::Future>,
         response_idx: usize,
     }
 }
 
-struct DispatcherState<S: Service, U: Encoder + Decoder> {
+bitflags::bitflags! {
+    struct Flags: u8  {
+        const READY_ERR  = 0b0001;
+        const IO_ERR     = 0b0010;
+    }
+}
+
+struct DispatcherInner {
+    io: IoBoxed,
+    keepalive_timeout: Cell<time::Duration>,
+}
+
+struct DispatcherState<S: Service<DispatchItem<U>>, U: Encoder + Decoder> {
     error: Option<IoDispatcherError<S::Error, <U as Encoder>::Error>>,
     base: usize,
     queue: VecDeque<ServiceResult<Result<S::Response, S::Error>>>,
@@ -65,90 +77,57 @@ enum IoDispatcherState {
 }
 
 pub(crate) enum IoDispatcherError<S, U> {
-    None,
     KeepAlive,
     Encoder(U),
     Service(S),
 }
 
-impl<S, U> From<Either<S, U>> for IoDispatcherError<S, U> {
-    fn from(err: Either<S, U>) -> Self {
-        match err {
-            Either::Left(err) => IoDispatcherError::Service(err),
-            Either::Right(err) => IoDispatcherError::Encoder(err),
-        }
+impl<S, U> From<S> for IoDispatcherError<S, U> {
+    fn from(err: S) -> Self {
+        IoDispatcherError::Service(err)
     }
 }
 
-impl<E1, E2: std::fmt::Debug> IoDispatcherError<E1, E2> {
-    fn take<U>(&mut self) -> Option<DispatchItem<U>>
-    where
-        U: Encoder<Error = E2> + Decoder,
-    {
-        match self {
-            IoDispatcherError::KeepAlive => {
-                *self = IoDispatcherError::None;
-                Some(DispatchItem::KeepAliveTimeout)
-            }
-            IoDispatcherError::Encoder(_) => {
-                let err = std::mem::replace(self, IoDispatcherError::None);
-                match err {
-                    IoDispatcherError::Encoder(err) => Some(DispatchItem::EncoderError(err)),
-                    _ => None,
-                }
-            }
-            IoDispatcherError::None | IoDispatcherError::Service(_) => None,
-        }
-    }
+fn insert_flags(flags: &mut Cell<Flags>, f: Flags) {
+    let mut val = flags.get();
+    val.insert(f);
+    flags.set(val);
 }
 
 impl<S, U> Dispatcher<S, U>
 where
-    S: Service<Request = DispatchItem<U>, Response = Option<Response<U>>> + 'static,
+    S: Service<DispatchItem<U>, Response = Option<Response<U>>> + 'static,
     U: Decoder + Encoder + Clone + 'static,
     <U as Encoder>::Item: 'static,
 {
     /// Construct new `Dispatcher` instance with outgoing messages stream.
-    pub(crate) fn with<T, F: IntoService<S>>(
-        io: T,
-        state: State,
+    pub(crate) fn new<F: IntoService<S, DispatchItem<U>>>(
+        io: IoBoxed,
         codec: U,
         service: F,
-        timer: Timer,
-    ) -> Self
-    where
-        T: AsyncRead + AsyncWrite + Unpin + 'static,
-    {
-        let updated = timer.now();
-        let keepalive_timeout = Seconds(30);
-        let io = Rc::new(RefCell::new(io));
+    ) -> Self {
+        let keepalive_timeout = Cell::new(Seconds(30).into());
 
         // register keepalive timer
-        let expire = updated + time::Duration::from(keepalive_timeout);
-        timer.register(expire, expire, &state);
+        io.start_keepalive_timer(keepalive_timeout.get());
 
-        let inner = Rc::new(RefCell::new(DispatcherState {
+        let state = Rc::new(RefCell::new(DispatcherState {
             error: None,
             base: 0,
             queue: VecDeque::new(),
         }));
-
-        // start support tasks
-        ntex::rt::spawn(ReadTask::new(io.clone(), state.clone()));
-        ntex::rt::spawn(WriteTask::new(io, state.clone()));
+        let pool = io.memory_pool().pool();
 
         Dispatcher {
+            state,
+            codec,
+            pool,
             st: IoDispatcherState::Processing,
             service: service.into_service(),
             response: None,
             response_idx: 0,
-            pool: state.memory_pool().pool(),
-            inner,
-            state,
-            codec,
-            timer,
-            updated,
-            keepalive_timeout,
+            flags: Cell::new(Flags::empty()),
+            inner: DispatcherInner { io, keepalive_timeout },
         }
     }
 
@@ -157,17 +136,10 @@ where
     /// To disable timeout set value to 0.
     ///
     /// By default keep-alive timeout is set to 30 seconds.
-    pub(crate) fn keepalive_timeout(mut self, timeout: Seconds) -> Self {
-        // register keepalive timer
-        let prev = self.updated + time::Duration::from(self.keepalive_timeout);
-        if timeout.is_zero() {
-            self.timer.unregister(prev, &self.state);
-        } else {
-            let expire = self.updated + time::Duration::from(timeout);
-            self.timer.register(expire, prev, &self.state);
-        }
-
-        self.keepalive_timeout = timeout;
+    pub(crate) fn keepalive_timeout(self, timeout: Seconds) -> Self {
+        let timeout = timeout.into();
+        self.inner.io.start_keepalive_timer(timeout);
+        self.inner.keepalive_timeout.set(timeout);
         self
     }
 
@@ -180,16 +152,27 @@ where
     ///
     /// By default disconnect timeout is set to 1 seconds.
     pub(crate) fn disconnect_timeout(self, val: Seconds) -> Self {
-        self.state.set_disconnect_timeout(val);
+        self.inner.io.set_disconnect_timeout(val.into());
         self
     }
 }
 
+impl DispatcherInner {
+    fn update_keepalive(&self) {
+        // update keep-alive timer
+        self.io.start_keepalive_timer(self.keepalive_timeout.get());
+    }
+
+    fn unregister_keepalive(&self) {
+        // unregister keep-alive timer
+        self.io.remove_keepalive_timer();
+        self.keepalive_timeout.set(time::Duration::ZERO);
+    }
+}
+
 impl<S, U> DispatcherState<S, U>
 where
-    S: Service<Request = DispatchItem<U>, Response = Option<Response<U>>>,
-    S::Error: 'static,
-    S::Future: 'static,
+    S: Service<DispatchItem<U>, Response = Option<Response<U>>> + 'static,
     U: Encoder + Decoder,
     <U as Encoder>::Item: 'static,
 {
@@ -197,7 +180,7 @@ where
         &mut self,
         item: Result<S::Response, S::Error>,
         response_idx: usize,
-        write: Write<'_>,
+        write: &IoRef,
         codec: &U,
         wake: bool,
     ) {
@@ -207,21 +190,37 @@ where
         if idx == 0 {
             let _ = self.queue.pop_front();
             self.base = self.base.wrapping_add(1);
-            if let Err(err) = write.encode_result(item, codec) {
-                self.error = Some(err.into());
+            match item {
+                Err(err) => {
+                    self.error = Some(err.into());
+                }
+                Ok(Some(item)) => {
+                    if let Err(err) = write.encode(item, codec) {
+                        self.error = Some(IoDispatcherError::Encoder(err));
+                    }
+                }
+                Ok(None) => (),
             }
 
             // check remaining response
             while let Some(item) = self.queue.front_mut().and_then(|v| v.take()) {
                 let _ = self.queue.pop_front();
                 self.base = self.base.wrapping_add(1);
-                if let Err(err) = write.encode_result(item, codec) {
-                    self.error = Some(err.into());
+                match item {
+                    Err(err) => {
+                        self.error = Some(err.into());
+                    }
+                    Ok(Some(item)) => {
+                        if let Err(err) = write.encode(item, codec) {
+                            self.error = Some(IoDispatcherError::Encoder(err));
+                        }
+                    }
+                    Ok(None) => (),
                 }
             }
 
             if wake && self.queue.is_empty() {
-                write.wake_dispatcher()
+                write.wake()
             }
         } else {
             self.queue[idx] = ServiceResult::Ready(item);
@@ -231,7 +230,7 @@ where
 
 impl<S, U> Future for Dispatcher<S, U>
 where
-    S: Service<Request = DispatchItem<U>, Response = Option<Response<U>>> + 'static,
+    S: Service<DispatchItem<U>, Response = Option<Response<U>>> + 'static,
     U: Decoder + Encoder + Clone + 'static,
     <U as Encoder>::Item: 'static,
 {
@@ -239,20 +238,19 @@ where
 
     fn poll(mut self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Self::Output> {
         let mut this = self.as_mut().project();
-        let read = this.state.read();
-        let write = this.state.write();
+        let io = &this.inner.io;
 
-        // log::trace!("IO-DISP poll :{:?}:", this.st);
+        // println!("IO-DISP poll :{:?}:", this.st);
 
         // handle service response future
         if let Some(fut) = this.response.as_mut().as_pin_mut() {
             match fut.poll(cx) {
                 Poll::Pending => (),
                 Poll::Ready(item) => {
-                    this.inner.borrow_mut().handle_result(
+                    this.state.borrow_mut().handle_result(
                         item,
                         *this.response_idx,
-                        write,
+                        io.as_ref(),
                         this.codec,
                         false,
                     );
@@ -263,113 +261,54 @@ where
 
         // handle memory pool pressure
         if this.pool.poll_ready(cx).is_pending() {
-            read.pause(cx.waker());
+            io.pause();
             return Poll::Pending;
         }
 
-        match this.st {
-            IoDispatcherState::Processing => {
-                loop {
-                    // log::trace!("IO-DISP state :{:?}:", this.state.flags());
-
+        loop {
+            match this.st {
+                IoDispatcherState::Processing => {
+                    // println!("IO-DISP state :{:?}:", io.flags());
                     match this.service.poll_ready(cx) {
                         Poll::Ready(Ok(_)) => {
-                            let mut retry = false;
+                            // decode incoming bytes stream
+                            let item = match ready!(io.poll_recv(this.codec, cx)) {
+                                Ok(el) => {
+                                    // update keep-alive timer
+                                    this.inner.update_keepalive();
 
-                            // service is ready, wake io read task
-                            read.resume();
-
-                            // check keepalive timeout
-                            if this.state.is_keepalive() {
-                                log::trace!("keepalive timeout");
-                                let mut inner = this.inner.borrow_mut();
-                                if inner.error.is_none() {
-                                    inner.error = Some(IoDispatcherError::KeepAlive);
-                                }
-                                this.state.dispatcher_stopped();
-                            }
-
-                            let item = if this.state.is_dispatcher_stopped() {
-                                retry = true;
-                                let mut inner = this.inner.borrow_mut();
-
-                                // unregister keep-alive timer
-                                if this.keepalive_timeout.non_zero() {
-                                    this.timer.unregister(
-                                        *this.updated
-                                            + time::Duration::from(*this.keepalive_timeout),
-                                        this.state,
-                                    );
-                                }
-
-                                // process unhandled data
-                                if let Ok(Some(el)) = read.decode(this.codec) {
                                     Some(DispatchItem::Item(el))
-                                } else {
-                                    log::trace!("dispatcher is instructed to stop");
-
-                                    // check for errors
-                                    let item = inner
-                                        .error
-                                        .as_mut()
-                                        .and_then(|err| err.take())
-                                        .or_else(|| {
-                                            this.state
-                                                .take_io_error()
-                                                .map(DispatchItem::IoError)
-                                        });
-                                    *this.st = IoDispatcherState::Stop;
-                                    item
                                 }
-                            } else {
-                                // decode incoming bytes stream
-                                if read.is_ready() {
-                                    match read.decode(this.codec) {
-                                        Ok(Some(el)) => {
-                                            // update keep-alive timer
-                                            if this.keepalive_timeout.non_zero() {
-                                                let updated = this.timer.now();
-                                                if updated != *this.updated {
-                                                    let ka = time::Duration::from(
-                                                        *this.keepalive_timeout,
-                                                    );
-                                                    this.timer.register(
-                                                        updated + ka,
-                                                        *this.updated + ka,
-                                                        this.state,
-                                                    );
-                                                    *this.updated = updated;
-                                                }
-                                            }
-
-                                            Some(DispatchItem::Item(el))
-                                        }
-                                        Ok(None) => {
-                                            // log::trace!("not enough data to decode next frame, register dispatch task");
-                                            read.wake(cx.waker());
-                                            return Poll::Pending;
-                                        }
-                                        Err(err) => {
-                                            retry = true;
-                                            *this.st = IoDispatcherState::Stop;
-
-                                            // unregister keep-alive timer
-                                            if this.keepalive_timeout.non_zero() {
-                                                this.timer.unregister(
-                                                    *this.updated
-                                                        + time::Duration::from(
-                                                            *this.keepalive_timeout,
-                                                        ),
-                                                    this.state,
-                                                );
-                                            }
-
-                                            Some(DispatchItem::DecoderError(err))
-                                        }
+                                Err(RecvError::Stop) => {
+                                    log::trace!("dispatcher is instructed to stop");
+                                    *this.st = IoDispatcherState::Stop;
+                                    None
+                                }
+                                Err(RecvError::KeepAlive) => {
+                                    // check keepalive timeout
+                                    log::trace!("keepalive timeout");
+                                    *this.st = IoDispatcherState::Stop;
+                                    let mut inner = this.state.borrow_mut();
+                                    if inner.error.is_none() {
+                                        inner.error = Some(IoDispatcherError::KeepAlive);
                                     }
-                                } else {
-                                    this.state.register_dispatcher(cx.waker());
-                                    return Poll::Pending;
+                                    Some(DispatchItem::KeepAliveTimeout)
+                                }
+                                Err(RecvError::WriteBackpressure) => {
+                                    if let Err(err) = ready!(io.poll_flush(cx, false)) {
+                                        *this.st = IoDispatcherState::Stop;
+                                        Some(DispatchItem::Disconnect(Some(err)))
+                                    } else {
+                                        continue;
+                                    }
+                                }
+                                Err(RecvError::Decoder(err)) => {
+                                    *this.st = IoDispatcherState::Stop;
+                                    Some(DispatchItem::DecoderError(err))
+                                }
+                                Err(RecvError::PeerGone(err)) => {
+                                    *this.st = IoDispatcherState::Stop;
+                                    Some(DispatchItem::Disconnect(err))
                                 }
                             };
 
@@ -381,17 +320,27 @@ where
                                     let res =
                                         this.response.as_mut().as_pin_mut().unwrap().poll(cx);
 
-                                    let mut inner = this.inner.borrow_mut();
+                                    let mut inner = this.state.borrow_mut();
                                     let response_idx =
                                         inner.base.wrapping_add(inner.queue.len() as usize);
 
                                     if let Poll::Ready(res) = res {
                                         // check if current result is only response atm
                                         if inner.queue.is_empty() {
-                                            if let Err(err) =
-                                                write.encode_result(res, this.codec)
-                                            {
-                                                inner.error = Some(err.into());
+                                            match res {
+                                                Err(err) => {
+                                                    inner.error = Some(err.into());
+                                                }
+                                                Ok(Some(item)) => {
+                                                    if let Err(err) =
+                                                        io.encode(item, this.codec)
+                                                    {
+                                                        inner.error = Some(
+                                                            IoDispatcherError::Encoder(err),
+                                                        );
+                                                    }
+                                                }
+                                                Ok(None) => (),
                                             }
                                         } else {
                                             *this.response_idx = response_idx;
@@ -403,97 +352,98 @@ where
                                         inner.queue.push_back(ServiceResult::Pending);
                                     }
                                 } else {
-                                    let mut inner = this.inner.borrow_mut();
+                                    let mut inner = this.state.borrow_mut();
                                     let response_idx =
                                         inner.base.wrapping_add(inner.queue.len() as usize);
                                     inner.queue.push_back(ServiceResult::Pending);
 
-                                    let st = this.state.clone();
+                                    let st = io.get_ref();
                                     let codec = this.codec.clone();
-                                    let inner = this.inner.clone();
+                                    let inner = this.state.clone();
                                     let fut = this.service.call(item);
                                     ntex::rt::spawn(async move {
                                         let item = fut.await;
                                         inner.borrow_mut().handle_result(
                                             item,
                                             response_idx,
-                                            st.write(),
+                                            &st,
                                             &codec,
                                             true,
                                         );
                                     });
                                 }
                             }
-
-                            // run again
-                            if retry {
-                                return self.poll(cx);
-                            }
                         }
                         Poll::Pending => {
                             // pause io read task
-                            log::trace!("service is not ready, register dispatch task");
-                            read.pause(cx.waker());
+                            log::trace!("service is not ready, pause read task");
+                            io.pause();
                             return Poll::Pending;
                         }
                         Poll::Ready(Err(err)) => {
                             log::trace!("service readiness check failed, stopping");
                             // service readiness error
                             *this.st = IoDispatcherState::Stop;
-                            this.inner.borrow_mut().error =
+                            this.state.borrow_mut().error =
                                 Some(IoDispatcherError::Service(err));
-                            this.state.dispatcher_ready_err();
-
-                            // unregister keep-alive timer
-                            if this.keepalive_timeout.non_zero() {
-                                this.timer.unregister(
-                                    *this.updated
-                                        + time::Duration::from(*this.keepalive_timeout),
-                                    this.state,
-                                );
-                            }
-
-                            return self.poll(cx);
+                            insert_flags(&mut *this.flags, Flags::READY_ERR);
                         }
                     }
                 }
-            }
-            // drain service responses
-            IoDispatcherState::Stop => {
-                // service may relay on poll_ready for response results
-                if !this.state.is_dispatcher_ready_err() {
-                    let _ = this.service.poll_ready(cx);
+                // drain service responses and shutdown io
+                IoDispatcherState::Stop => {
+                    this.inner.unregister_keepalive();
+
+                    // service may relay on poll_ready for response results
+                    if !this.flags.get().contains(Flags::READY_ERR) {
+                        let _ = this.service.poll_ready(cx);
+                    }
+
+                    if this.state.borrow().queue.is_empty() {
+                        if io.poll_shutdown(cx).is_ready() {
+                            log::trace!("io shutdown completed");
+                            *this.st = IoDispatcherState::Shutdown;
+                            continue;
+                        }
+                    } else if !this.flags.get().contains(Flags::IO_ERR) {
+                        match ready!(this.inner.io.poll_status_update(cx)) {
+                            IoStatusUpdate::PeerGone(_)
+                            | IoStatusUpdate::Stop
+                            | IoStatusUpdate::KeepAlive => {
+                                insert_flags(&mut *this.flags, Flags::IO_ERR);
+                                continue;
+                            }
+                            IoStatusUpdate::WriteBackpressure => {
+                                if ready!(this.inner.io.poll_flush(cx, true)).is_err() {
+                                    insert_flags(&mut *this.flags, Flags::IO_ERR);
+                                }
+                                continue;
+                            }
+                        }
+                    }
+                    return Poll::Pending;
                 }
+                // shutdown service
+                IoDispatcherState::Shutdown => {
+                    let is_err = this.state.borrow().error.is_some();
 
-                if this.inner.borrow().queue.is_empty() {
-                    this.state.shutdown_io();
-                    *this.st = IoDispatcherState::Shutdown;
-                    self.poll(cx)
-                } else {
-                    this.state.register_dispatcher(cx.waker());
-                    Poll::Pending
+                    return if this.service.poll_shutdown(cx, is_err).is_ready() {
+                        log::trace!("service shutdown is completed, stop");
+
+                        Poll::Ready(
+                            if let Some(IoDispatcherError::Service(err)) =
+                                this.state.borrow_mut().error.take()
+                            {
+                                Err(err)
+                            } else {
+                                Ok(())
+                            },
+                        )
+                    } else {
+                        Poll::Pending
+                    };
                 }
             }
-            // shutdown service
-            IoDispatcherState::Shutdown => {
-                let is_err = this.inner.borrow().error.is_some();
-
-                return if this.service.poll_shutdown(cx, is_err).is_ready() {
-                    log::trace!("service shutdown is completed, stop");
-
-                    Poll::Ready(
-                        if let Some(IoDispatcherError::Service(err)) =
-                            this.inner.borrow_mut().error.take()
-                        {
-                            Err(err)
-                        } else {
-                            Ok(())
-                        },
-                    )
-                } else {
-                    Poll::Pending
-                };
-            }
         }
     }
 }
@@ -504,6 +454,7 @@ mod tests {
 
     use ntex::channel::condition::Condition;
     use ntex::codec::BytesCodec;
+    use ntex::io as nio;
     use ntex::testing::Io;
     use ntex::time::{sleep, Millis};
     use ntex::util::{Bytes, Ready};
@@ -512,48 +463,43 @@ mod tests {
 
     impl<S, U> Dispatcher<S, U>
     where
-        S: Service<Request = DispatchItem<U>, Response = Option<Response<U>>>,
+        S: Service<DispatchItem<U>, Response = Option<Response<U>>>,
         S::Error: 'static,
         S::Future: 'static,
         U: Decoder + Encoder + 'static,
         <U as Encoder>::Item: 'static,
     {
         /// Construct new `Dispatcher` instance
-        pub(crate) fn new<T, F: IntoService<S>>(
-            io: T,
+        pub(crate) fn new_debug<F: IntoService<S, DispatchItem<U>>>(
+            io: Io,
             codec: U,
-            state: State,
             service: F,
-        ) -> Self
-        where
-            T: AsyncRead + AsyncWrite + Unpin + 'static,
-        {
-            let timer = Timer::new(Millis::ONE_SEC);
-            let keepalive_timeout = Seconds(30);
-            let updated = timer.now();
-            let io = Rc::new(RefCell::new(io));
-            ntex::rt::spawn(ReadTask::new(io.clone(), state.clone()));
-            ntex::rt::spawn(WriteTask::new(io.clone(), state.clone()));
+        ) -> (Self, nio::IoRef) {
+            let keepalive_timeout = Cell::new(Seconds(30).into());
+            let io = nio::Io::new(io).seal();
+            io.start_keepalive_timer(keepalive_timeout.get());
+            let rio = io.get_ref();
 
-            let inner = Rc::new(RefCell::new(DispatcherState {
+            let state = Rc::new(RefCell::new(DispatcherState {
                 error: None,
                 base: 0,
                 queue: VecDeque::new(),
             }));
 
-            Dispatcher {
-                service: service.into_service(),
-                st: IoDispatcherState::Processing,
-                pool: state.memory_pool().pool(),
-                response: None,
-                response_idx: 0,
-                inner,
-                state,
-                timer,
-                codec,
-                updated,
-                keepalive_timeout,
-            }
+            (
+                Dispatcher {
+                    state,
+                    codec,
+                    service: service.into_service(),
+                    st: IoDispatcherState::Processing,
+                    response: None,
+                    response_idx: 0,
+                    pool: io.memory_pool().pool(),
+                    flags: Cell::new(Flags::empty()),
+                    inner: DispatcherInner { keepalive_timeout, io: IoBoxed::from(io) },
+                },
+                rio,
+            )
         }
     }
 
@@ -563,10 +509,9 @@ mod tests {
         client.remote_buffer_cap(1024);
         client.write("GET /test HTTP/1\r\n\r\n");
 
-        let disp = Dispatcher::new(
+        let (disp, _) = Dispatcher::new_debug(
             server,
             BytesCodec,
-            State::new(),
             ntex::service::fn_service(|msg: DispatchItem<BytesCodec>| async move {
                 sleep(Millis(50)).await;
                 if let DispatchItem::Item(msg) = msg {
@@ -601,10 +546,9 @@ mod tests {
         let condition = Condition::new();
         let waiter = condition.wait();
 
-        let disp = Dispatcher::new(
+        let (disp, _) = Dispatcher::new_debug(
             server,
             BytesCodec,
-            State::new(),
             ntex::service::fn_service(move |msg: DispatchItem<BytesCodec>| {
                 let waiter = waiter.clone();
                 async move {
@@ -641,11 +585,9 @@ mod tests {
         client.remote_buffer_cap(1024);
         client.write("GET /test HTTP/1\r\n\r\n");
 
-        let st = State::new();
-        let disp = Dispatcher::new(
+        let (disp, io) = Dispatcher::new_debug(
             server,
             BytesCodec,
-            st.clone(),
             ntex::service::fn_service(|msg: DispatchItem<BytesCodec>| async move {
                 if let DispatchItem::Item(msg) = msg {
                     Ok::<_, ()>(Some(msg.freeze()))
@@ -661,11 +603,11 @@ mod tests {
         let buf = client.read().await.unwrap();
         assert_eq!(buf, Bytes::from_static(b"GET /test HTTP/1\r\n\r\n"));
 
-        assert!(st.write().encode(Bytes::from_static(b"test"), &BytesCodec).is_ok());
+        assert!(io.encode(Bytes::from_static(b"test"), &BytesCodec).is_ok());
         let buf = client.read().await.unwrap();
         assert_eq!(buf, Bytes::from_static(b"test"));
 
-        st.close();
+        io.close();
         sleep(Millis(1200)).await;
         assert!(client.is_server_dropped());
     }
@@ -676,11 +618,9 @@ mod tests {
         client.remote_buffer_cap(0);
         client.write("GET /test HTTP/1\r\n\r\n");
 
-        let state = State::new();
-        let disp = Dispatcher::new(
+        let (disp, io) = Dispatcher::new_debug(
             server,
             BytesCodec,
-            state.clone(),
             ntex::service::fn_service(|_: DispatchItem<BytesCodec>| async move {
                 Err::<Option<Bytes>, _>(())
             }),
@@ -689,10 +629,7 @@ mod tests {
             let _ = disp.await;
         });
 
-        state
-            .write()
-            .encode(Bytes::from_static(b"GET /test HTTP/1\r\n\r\n"), &BytesCodec)
-            .unwrap();
+        io.encode(Bytes::from_static(b"GET /test HTTP/1\r\n\r\n"), &BytesCodec).unwrap();
 
         // buffer should be flushed
         client.remote_buffer_cap(1024);
@@ -700,7 +637,7 @@ mod tests {
         assert_eq!(buf, Bytes::from_static(b"GET /test HTTP/1\r\n\r\n"));
 
         // write side must be closed, dispatcher waiting for read side to close
-        assert!(client.is_closed());
+        assert!(!client.is_closed());
 
         // close read side
         client.close().await;
@@ -717,8 +654,7 @@ mod tests {
 
         struct Srv(Rc<Cell<usize>>);
 
-        impl Service for Srv {
-            type Request = DispatchItem<BytesCodec>;
+        impl Service<DispatchItem<BytesCodec>> for Srv {
             type Response = Option<Response<BytesCodec>>;
             type Error = ();
             type Future = Ready<Option<Response<BytesCodec>>, ()>;
@@ -733,12 +669,8 @@ mod tests {
             }
         }
 
-        let state = State::new();
-        let disp = Dispatcher::new(server, BytesCodec, state.clone(), Srv(counter.clone()));
-        state
-            .write()
-            .encode(Bytes::from_static(b"GET /test HTTP/1\r\n\r\n"), &mut BytesCodec)
-            .unwrap();
+        let (disp, io) = Dispatcher::new_debug(server, BytesCodec, Srv(counter.clone()));
+        io.encode(Bytes::from_static(b"GET /test HTTP/1\r\n\r\n"), &mut BytesCodec).unwrap();
         ntex::rt::spawn(async move {
             let _ = disp.await;
         });
