@@ -1,9 +1,9 @@
 use std::num::{NonZeroU16, NonZeroU32};
 use std::task::{Context, Poll};
-use std::{convert::TryFrom, future::Future, io::Cursor, pin::Pin};
+use std::{convert::TryFrom, future::Future, io::Cursor, marker::PhantomData, pin::Pin};
 
 use ntex::service::Service;
-use ntex::util::{Buf, BufMut, ByteString, Bytes, BytesMut};
+use ntex::util::{Buf, BufMut, ByteString, Bytes, BytesMut, Either};
 
 use crate::error::{DecodeError, EncodeError};
 
@@ -306,15 +306,15 @@ pub(crate) fn write_variable_length(len: u32, dst: &mut BytesMut) {
 }
 
 /// Check service readiness
-pub(crate) fn ready<S>(service: &S) -> Ready<'_, S> {
-    Ready(service)
+pub(crate) fn ready<S, R>(service: &S) -> Ready<'_, S, R> {
+    Ready(service, PhantomData)
 }
 
-pub(crate) struct Ready<'a, S>(&'a S);
+pub(crate) struct Ready<'a, S, R>(&'a S, PhantomData<R>);
 
-impl<'a, S> Unpin for Ready<'a, S> {}
+impl<'a, S, R> Unpin for Ready<'a, S, R> {}
 
-impl<'a, S: Service> Future for Ready<'a, S> {
+impl<'a, S: Service<R>, R> Future for Ready<'a, S, R> {
     type Output = Result<(), S::Error>;
 
     fn poll(self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Self::Output> {
@@ -322,6 +322,43 @@ impl<'a, S: Service> Future for Ready<'a, S> {
     }
 }
 
+pub(crate) async fn select<F1, F2>(fut1: F1, fut2: F2) -> Either<F1::Output, F2::Output>
+where
+    F1: Future,
+    F2: Future,
+{
+    Select { fut1, fut2 }.await
+}
+
+pin_project_lite::pin_project! {
+    struct Select<F1, F2>{
+        #[pin]
+        fut1: F1,
+        #[pin]
+        fut2: F2
+    }
+}
+
+impl<F1, F2> Future for Select<F1, F2>
+where
+    F1: Future,
+    F2: Future,
+{
+    type Output = Either<F1::Output, F2::Output>;
+
+    fn poll(self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Self::Output> {
+        let this = self.project();
+
+        if let Poll::Ready(res) = this.fut1.poll(cx) {
+            Poll::Ready(Either::Left(res))
+        } else if let Poll::Ready(res) = this.fut2.poll(cx) {
+            Poll::Ready(Either::Right(res))
+        } else {
+            Poll::Pending
+        }
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
