@@ -1,4 +1,8 @@
-pub use crate::v3::control::{Closed, ControlResult, Disconnect, Error, ProtocolError};
+use std::io;
+
+pub use crate::v3::control::{
+    Closed, ControlResult, Disconnect, Error, PeerGone, ProtocolError,
+};
 use crate::v3::{codec, control::ControlResultKind, error};
 
 pub enum ControlMessage<E> {
@@ -12,6 +16,8 @@ pub enum ControlMessage<E> {
     Error(Error<E>),
     /// Protocol level error
     ProtocolError(ProtocolError),
+    /// Peer is gone
+    PeerGone(PeerGone),
 }
 
 impl<E> ControlMessage<E> {
@@ -35,6 +41,10 @@ impl<E> ControlMessage<E> {
         ControlMessage::ProtocolError(ProtocolError::new(err))
     }
 
+    pub(super) fn peer_gone(err: Option<io::Error>) -> Self {
+        ControlMessage::PeerGone(PeerGone(err))
+    }
+
     pub fn disconnect(&self) -> ControlResult {
         ControlResult { result: ControlResultKind::Disconnect }
     }
