@@ -5,7 +5,7 @@ use std::num::{NonZeroU16, NonZeroU32};
 use super::ack_props;
 use crate::error::{DecodeError, EncodeError};
 use crate::types::QoS;
-use crate::utils::{self, Decode, Encode};
+use crate::utils::{self, write_variable_length, Decode, Encode};
 use crate::v5::codec::{encode::*, property_type as pt, UserProperties, UserProperty};
 
 /// Represents SUBSCRIBE packet
@@ -191,7 +191,12 @@ impl EncodeLtd for Subscribe {
         let prop_len = self.id.map_or(0, |v| var_int_len(v.get() as usize))
             + self.user_properties.encoded_size() as u32; // safe: size was already checked against maximum
         utils::write_variable_length(prop_len, buf);
-        encode_property(&self.id, pt::SUB_ID, buf)?;
+
+        if let Some(id) = self.id {
+            buf.put_u8(pt::SUB_ID);
+            write_variable_length(id.get(), buf);
+        }
+
         for (filter, opts) in self.topic_filters.iter() {
             filter.encode(buf)?;
             opts.encode(buf)?;
@@ -312,8 +317,8 @@ mod tests {
             status: Vec::new(),
         };
 
-        let mut buf = BytesMut::new();
         let size = ack.encoded_size(99999);
+        let mut buf = BytesMut::with_capacity(size);
         ack.encode(&mut buf, size as u32).unwrap();
         assert_eq!(ack, SubscribeAck::decode(&mut buf.freeze()).unwrap());
 
@@ -323,8 +328,8 @@ mod tests {
             reason_string: None,
             status: vec![SubscribeAckReason::GrantedQos0],
         };
-        let mut buf = BytesMut::new();
         let size = ack.encoded_size(99999);
+        let mut buf = BytesMut::with_capacity(size);
         ack.encode(&mut buf, size as u32).unwrap();
         assert_eq!(ack, SubscribeAck::decode(&mut buf.freeze()).unwrap());
 
@@ -345,8 +350,8 @@ mod tests {
             reason_string: None,
             status: vec![UnsubscribeAckReason::Success],
         };
-        let mut buf = BytesMut::new();
         let size = ack.encoded_size(99999);
+        let mut buf = BytesMut::with_capacity(size);
         ack.encode(&mut buf, size as u32).unwrap();
         assert_eq!(ack, UnsubscribeAck::decode(&mut buf.freeze()).unwrap());
     }
