@@ -1,15 +1,13 @@
 use std::task::{Context, Poll};
-use std::{cell::RefCell, convert::TryFrom, fmt, future::Future, marker, pin::Pin, rc::Rc};
+use std::{convert::TryFrom, fmt, future::Future, marker, pin::Pin, rc::Rc};
 
-use ntex::codec::{AsyncRead, AsyncWrite};
-use ntex::framed::WriteTask;
+use ntex::io::{DispatchItem, IoBoxed};
 use ntex::service::{IntoServiceFactory, Service, ServiceFactory};
 use ntex::time::{Millis, Seconds, Sleep};
-use ntex::util::timeout::{Timeout, TimeoutError};
-use ntex::util::{Either, PoolId, PoolRef};
+use ntex::util::{timeout::Timeout, timeout::TimeoutError, Either};
 
 use crate::error::{MqttError, ProtocolError};
-use crate::io::{DispatchItem, Dispatcher, State, Timer};
+use crate::io::Dispatcher;
 use crate::service::{FramedService, FramedService2};
 use crate::types::QoS;
 
@@ -22,7 +20,7 @@ use super::shared::{MqttShared, MqttSinkPool};
 use super::{codec as mqtt, dispatcher::factory, MqttSink, Session};
 
 /// Mqtt Server
-pub struct MqttServer<Io, St, C: ServiceFactory, Cn: ServiceFactory, P: ServiceFactory> {
+pub struct MqttServer<St, C, Cn, P> {
     handshake: C,
     srv_control: Cn,
     srv_publish: P,
@@ -33,27 +31,20 @@ pub struct MqttServer<Io, St, C: ServiceFactory, Cn: ServiceFactory, P: ServiceF
     disconnect_timeout: Seconds,
     max_topic_alias: u16,
     pub(super) pool: Rc<MqttSinkPool>,
-    _t: marker::PhantomData<(Io, St)>,
+    _t: marker::PhantomData<St>,
 }
 
-impl<Io, St, C>
-    MqttServer<
-        Io,
-        St,
-        C,
-        DefaultControlService<St, C::Error>,
-        DefaultPublishService<St, C::Error>,
-    >
+impl<St, C>
+    MqttServer<St, C, DefaultControlService<St, C::Error>, DefaultPublishService<St, C::Error>>
 where
     St: 'static,
-    C: ServiceFactory<Config = (), Request = Handshake<Io>, Response = HandshakeAck<Io, St>>
-        + 'static,
+    C: ServiceFactory<Handshake, Response = HandshakeAck<St>> + 'static,
     C::Error: fmt::Debug,
 {
     /// Create server factory and provide handshake service
     pub fn new<F>(handshake: F) -> Self
     where
-        F: IntoServiceFactory<C>,
+        F: IntoServiceFactory<C, Handshake>,
     {
         MqttServer {
             handshake: handshake.into_factory(),
@@ -71,19 +62,14 @@ where
     }
 }
 
-impl<Io, St, C, Cn, P> MqttServer<Io, St, C, Cn, P>
+impl<St, C, Cn, P> MqttServer<St, C, Cn, P>
 where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
     St: 'static,
-    C: ServiceFactory<Config = (), Request = Handshake<Io>, Response = HandshakeAck<Io, St>>
-        + 'static,
+    C: ServiceFactory<Handshake, Response = HandshakeAck<St>> + 'static,
     C::Error: fmt::Debug,
-    Cn: ServiceFactory<
-            Config = Session<St>,
-            Request = ControlMessage<C::Error>,
-            Response = ControlResult,
-        > + 'static,
-    P: ServiceFactory<Config = Session<St>, Request = Publish, Response = PublishAck> + 'static,
+    Cn: ServiceFactory<ControlMessage<C::Error>, Session<St>, Response = ControlResult>
+        + 'static,
+    P: ServiceFactory<Publish, Session<St>, Response = PublishAck> + 'static,
 {
     /// Set handshake timeout.
     ///
@@ -141,27 +127,15 @@ where
         self
     }
 
-    /// Set memory pool.
-    ///
-    /// Use specified memory pool for memory allocations. By default P5
-    /// memory pool is used.
-    pub fn memory_pool(self, id: PoolId) -> Self {
-        self.pool.pool.set(id.pool_ref());
-        self
-    }
-
     /// Service to handle control packets
     ///
     /// All control packets are processed sequentially, max number of buffered
     /// control packets is 16.
-    pub fn control<F, Srv>(self, service: F) -> MqttServer<Io, St, C, Srv, P>
+    pub fn control<F, Srv>(self, service: F) -> MqttServer<St, C, Srv, P>
     where
-        F: IntoServiceFactory<Srv>,
-        Srv: ServiceFactory<
-                Config = Session<St>,
-                Request = ControlMessage<C::Error>,
-                Response = ControlResult,
-            > + 'static,
+        F: IntoServiceFactory<Srv, ControlMessage<C::Error>, Session<St>>,
+        Srv: ServiceFactory<ControlMessage<C::Error>, Session<St>, Response = ControlResult>
+            + 'static,
         C::Error: From<Srv::Error> + From<Srv::InitError>,
     {
         MqttServer {
@@ -180,12 +154,11 @@ where
     }
 
     /// Set service to handle publish packets and create mqtt server factory
-    pub fn publish<F, Srv>(self, publish: F) -> MqttServer<Io, St, C, Cn, Srv>
+    pub fn publish<F, Srv>(self, publish: F) -> MqttServer<St, C, Cn, Srv>
     where
-        F: IntoServiceFactory<Srv> + 'static,
+        F: IntoServiceFactory<Srv, Publish, Session<St>>,
         C::Error: From<Srv::Error> + From<Srv::InitError>,
-        Srv: ServiceFactory<Config = Session<St>, Request = Publish, Response = PublishAck>
-            + 'static,
+        Srv: ServiceFactory<Publish, Session<St>, Response = PublishAck> + 'static,
         Srv::Error: fmt::Debug,
         PublishAck: TryFrom<Srv::Error, Error = C::Error>,
     {
@@ -205,32 +178,25 @@ where
     }
 }
 
-impl<Io, St, C, Cn, P> MqttServer<Io, St, C, Cn, P>
+impl<St, C, Cn, P> MqttServer<St, C, Cn, P>
 where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
     St: 'static,
-    C: ServiceFactory<Config = (), Request = Handshake<Io>, Response = HandshakeAck<Io, St>>
-        + 'static,
+    C: ServiceFactory<Handshake, Response = HandshakeAck<St>> + 'static,
     C::Error: From<Cn::Error>
         + From<Cn::InitError>
         + From<P::Error>
         + From<P::InitError>
         + fmt::Debug,
-    Cn: ServiceFactory<
-            Config = Session<St>,
-            Request = ControlMessage<C::Error>,
-            Response = ControlResult,
-        > + 'static,
-    P: ServiceFactory<Config = Session<St>, Request = Publish, Response = PublishAck> + 'static,
+    Cn: ServiceFactory<ControlMessage<C::Error>, Session<St>, Response = ControlResult>
+        + 'static,
+    P: ServiceFactory<Publish, Session<St>, Response = PublishAck> + 'static,
     P::Error: fmt::Debug,
     PublishAck: TryFrom<P::Error, Error = C::Error>,
 {
     /// Set service to handle publish packets and create mqtt server factory
     pub fn finish(
         self,
-    ) -> impl ServiceFactory<Config = (), Request = Io, Response = (), Error = MqttError<C::Error>>
-    {
-        let pool = self.pool.pool.get().pool();
+    ) -> impl ServiceFactory<IoBoxed, Response = (), Error = MqttError<C::Error>> {
         let handshake = self.handshake;
         let publish = self.srv_publish.map_init_err(|e| MqttError::Service(e.into()));
         let control = self
@@ -249,7 +215,6 @@ where
                 self.pool,
             ),
             factory(publish, control),
-            pool,
             self.disconnect_timeout,
         )
     }
@@ -258,13 +223,11 @@ where
     pub(crate) fn inner_finish(
         self,
     ) -> impl ServiceFactory<
-        Config = (),
-        Request = (Io, State, Option<Sleep>),
+        (IoBoxed, Option<Sleep>),
         Response = (),
         Error = MqttError<C::Error>,
         InitError = C::InitError,
     > {
-        let pool = self.pool.pool.get().pool();
         let handshake = self.handshake;
         let publish = self.srv_publish.map_init_err(|e| MqttError::Service(e.into()));
         let control = self
@@ -273,7 +236,7 @@ where
             .map_init_err(|e| MqttError::Service(e.into()));
 
         FramedService2::new(
-            handshake_service_factory2(
+            handshake_service_factory(
                 handshake,
                 self.max_size,
                 self.max_receive,
@@ -283,7 +246,6 @@ where
                 self.pool,
             ),
             factory(publish, control),
-            pool,
             self.disconnect_timeout,
         )
     }
@@ -293,14 +255,13 @@ where
         self,
         check: F,
     ) -> impl ServiceFactory<
-        Config = (),
-        Request = SelectItem<Io>,
-        Response = Either<SelectItem<Io>, ()>,
+        SelectItem,
+        Response = Either<SelectItem, ()>,
         Error = MqttError<C::Error>,
         InitError = C::InitError,
     >
     where
-        F: Fn(&Handshake<Io>) -> R + 'static,
+        F: Fn(&Handshake) -> R + 'static,
         R: Future<Output = Result<bool, C::Error>> + 'static,
     {
         let publish = self.srv_publish.map_init_err(|e| MqttError::Service(e.into()));
@@ -309,7 +270,7 @@ where
             .map_err(<C::Error>::from)
             .map_init_err(|e| MqttError::Service(e.into()));
 
-        ServerSelector::<St, _, _, Io, _, _> {
+        ServerSelector::<St, _, _, _, _> {
             check: Rc::new(check),
             connect: self.handshake,
             handler: Rc::new(factory(publish, control)),
@@ -318,13 +279,12 @@ where
             max_topic_alias: self.max_topic_alias,
             max_qos: self.max_qos,
             disconnect_timeout: self.disconnect_timeout,
-            time: Timer::new(Millis::ONE_SEC),
             _t: marker::PhantomData,
         }
     }
 }
 
-fn handshake_service_factory<Io, St, C>(
+fn handshake_service_factory<St, C>(
     factory: C,
     max_size: u32,
     max_receive: u16,
@@ -333,94 +293,36 @@ fn handshake_service_factory<Io, St, C>(
     handshake_timeout: Seconds,
     pool: Rc<MqttSinkPool>,
 ) -> impl ServiceFactory<
-    Config = (),
-    Request = Io,
-    Response = (Io, State, Rc<MqttShared>, Session<St>, Seconds),
-    Error = MqttError<C::Error>,
->
-where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
-    C: ServiceFactory<Config = (), Request = Handshake<Io>, Response = HandshakeAck<Io, St>>,
-    C::Error: fmt::Debug,
-{
-    ntex::service::apply(
-        Timeout::new(Millis::from(handshake_timeout)),
-        ntex::service::fn_factory(move || {
-            let pool = pool.clone();
-
-            let fut = factory.new_service(());
-            async move {
-                let service = fut.await?;
-                let pool = pool.clone();
-                let service = Rc::new(service.map_err(MqttError::Service));
-                Ok::<_, C::InitError>(ntex::service::apply_fn(
-                    service,
-                    move |io: Io, service| {
-                        handshake(
-                            io,
-                            None,
-                            service.clone(),
-                            max_size,
-                            max_receive,
-                            max_topic_alias,
-                            max_qos,
-                            pool.clone(),
-                        )
-                    },
-                ))
-            }
-        }),
-    )
-    .map_err(|e| match e {
-        TimeoutError::Service(e) => e,
-        TimeoutError::Timeout => MqttError::HandshakeTimeout,
-    })
-}
-
-fn handshake_service_factory2<Io, St, C>(
-    factory: C,
-    max_size: u32,
-    max_receive: u16,
-    max_topic_alias: u16,
-    max_qos: Option<QoS>,
-    handshake_timeout: Seconds,
-    pool: Rc<MqttSinkPool>,
-) -> impl ServiceFactory<
-    Config = (),
-    Request = (Io, State),
-    Response = (Io, State, Rc<MqttShared>, Session<St>, Seconds),
+    IoBoxed,
+    Response = (IoBoxed, Rc<MqttShared>, Session<St>, Seconds),
     Error = MqttError<C::Error>,
     InitError = C::InitError,
 >
 where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
-    C: ServiceFactory<Config = (), Request = Handshake<Io>, Response = HandshakeAck<Io, St>>,
+    C: ServiceFactory<Handshake, Response = HandshakeAck<St>>,
     C::Error: fmt::Debug,
 {
     ntex::service::apply(
         Timeout::new(Millis::from(handshake_timeout)),
         ntex::service::fn_factory(move || {
             let pool = pool.clone();
+
             let fut = factory.new_service(());
             async move {
                 let service = fut.await?;
                 let pool = pool.clone();
                 let service = Rc::new(service.map_err(MqttError::Service));
-                Ok::<_, C::InitError>(ntex::service::apply_fn(
-                    service,
-                    move |(io, state), service| {
-                        handshake(
-                            io,
-                            Some(state),
-                            service.clone(),
-                            max_size,
-                            max_receive,
-                            max_topic_alias,
-                            max_qos,
-                            pool.clone(),
-                        )
-                    },
-                ))
+                Ok::<_, C::InitError>(ntex::service::apply_fn(service, move |io, service| {
+                    handshake(
+                        io,
+                        service.clone(),
+                        max_size,
+                        max_receive,
+                        max_topic_alias,
+                        max_qos,
+                        pool.clone(),
+                    )
+                }))
             }
         }),
     )
@@ -431,41 +333,36 @@ where
 }
 
 #[allow(clippy::too_many_arguments)]
-async fn handshake<Io, S, St, E>(
-    mut io: Io,
-    state: Option<State>,
+async fn handshake<S, St, E>(
+    io: IoBoxed,
     service: S,
     max_size: u32,
     mut max_receive: u16,
     mut max_topic_alias: u16,
     max_qos: Option<QoS>,
     pool: Rc<MqttSinkPool>,
-) -> Result<(Io, State, Rc<MqttShared>, Session<St>, Seconds), S::Error>
+) -> Result<(IoBoxed, Rc<MqttShared>, Session<St>, Seconds), S::Error>
 where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
-    S: Service<Request = Handshake<Io>, Response = HandshakeAck<Io, St>, Error = MqttError<E>>,
+    S: Service<Handshake, Response = HandshakeAck<St>, Error = MqttError<E>>,
 {
     log::trace!("Starting mqtt v5 handshake");
 
-    let state = state.unwrap_or_else(|| State::with_memory_pool(pool.pool.get()));
-    let shared = Rc::new(MqttShared::new(state.clone(), mqtt::Codec::default(), 0, pool));
+    let shared = Rc::new(MqttShared::new(io.get_ref(), mqtt::Codec::default(), 0, pool));
 
     // set max inbound (decoder) packet size
     shared.codec.set_max_inbound_size(max_size);
 
     // read first packet
-    let packet = state
-        .next(&mut io, &shared.codec)
+    let packet = io
+        .recv(&shared.codec)
         .await
         .map_err(|err| {
             log::trace!("Error is received during mqtt handshake: {:?}", err);
             MqttError::from(err)
-        })
-        .and_then(|res| {
-            res.ok_or_else(|| {
-                log::trace!("Server mqtt is disconnected during handshake");
-                MqttError::Disconnected
-            })
+        })?
+        .ok_or_else(|| {
+            log::trace!("Server mqtt is disconnected during handshake");
+            MqttError::Disconnected(None)
         })?;
 
     match packet {
@@ -515,17 +412,12 @@ where
                         ack.packet.server_keepalive_sec = Some(ack.keepalive as u16);
                     }
 
-                    state
-                        .send(
-                            &mut ack.io,
-                            &shared.codec,
-                            mqtt::Packet::ConnectAck(Box::new(ack.packet)),
-                        )
+                    ack.io
+                        .send(mqtt::Packet::ConnectAck(Box::new(ack.packet)), &shared.codec)
                         .await?;
 
                     Ok((
                         ack.io,
-                        shared.state.clone(),
                         shared.clone(),
                         Session::new_v5(
                             session,
@@ -539,24 +431,11 @@ where
                 None => {
                     log::trace!("Failed to complete handshake: {:#?}", ack.packet);
 
-                    if ack.shared.state.is_open()
-                        && ack
-                            .shared
-                            .state
-                            .write()
-                            .encode(
-                                mqtt::Packet::ConnectAck(Box::new(ack.packet)),
-                                &ack.shared.codec,
-                            )
-                            .is_ok()
-                    {
-                        WriteTask::shutdown(
-                            Rc::new(RefCell::new(ack.io)),
-                            ack.shared.state.clone(),
-                        )
-                        .await;
-                    }
-                    Err(MqttError::Disconnected)
+                    ack.io
+                        .send(mqtt::Packet::ConnectAck(Box::new(ack.packet)), &ack.shared.codec)
+                        .await?;
+                    let _ = ack.io.shutdown().await;
+                    Err(MqttError::Disconnected(None))
                 }
             }
         }
@@ -570,47 +449,42 @@ where
     }
 }
 
-pub(crate) struct ServerSelector<St, C, T, Io, F, R> {
+pub(crate) struct ServerSelector<St, C, T, F, R> {
     connect: C,
     handler: Rc<T>,
-    time: Timer,
     check: Rc<F>,
     max_size: u32,
     max_receive: u16,
     max_qos: Option<QoS>,
     disconnect_timeout: Seconds,
     max_topic_alias: u16,
-    _t: marker::PhantomData<(St, Io, R)>,
+    _t: marker::PhantomData<(St, R)>,
 }
 
-impl<St, C, T, Io, F, R> ServiceFactory for ServerSelector<St, C, T, Io, F, R>
+impl<St, C, T, F, R> ServiceFactory<SelectItem> for ServerSelector<St, C, T, F, R>
 where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
-    F: Fn(&Handshake<Io>) -> R + 'static,
+    St: 'static,
+    F: Fn(&Handshake) -> R + 'static,
     R: Future<Output = Result<bool, C::Error>>,
-    C: ServiceFactory<Config = (), Request = Handshake<Io>, Response = HandshakeAck<Io, St>>
-        + 'static,
+    C: ServiceFactory<Handshake, Response = HandshakeAck<St>> + 'static,
     C::Error: fmt::Debug,
     T: ServiceFactory<
-            Config = Session<St>,
-            Request = DispatchItem<Rc<MqttShared>>,
+            DispatchItem<Rc<MqttShared>>,
+            Session<St>,
             Response = Option<mqtt::Packet>,
             Error = MqttError<C::Error>,
             InitError = MqttError<C::Error>,
         > + 'static,
 {
-    type Config = ();
-    type Request = SelectItem<Io>;
-    type Response = Either<SelectItem<Io>, ()>;
+    type Response = Either<SelectItem, ()>;
     type Error = MqttError<C::Error>;
     type InitError = C::InitError;
-    type Service = ServerSelectorImpl<St, C::Service, T, Io, F, R>;
+    type Service = ServerSelectorImpl<St, C::Service, T, F, R>;
     type Future = Pin<Box<dyn Future<Output = Result<Self::Service, Self::InitError>>>>;
 
     fn new_service(&self, _: ()) -> Self::Future {
         let fut = self.connect.new_service(());
         let handler = self.handler.clone();
-        let time = self.time.clone();
         let check = self.check.clone();
         let max_size = self.max_size;
         let max_receive = self.max_receive;
@@ -622,7 +496,6 @@ where
         Box::pin(async move {
             Ok(ServerSelectorImpl {
                 handler,
-                time,
                 check,
                 max_size,
                 max_receive,
@@ -636,7 +509,7 @@ where
     }
 }
 
-pub(crate) struct ServerSelectorImpl<St, C, T, Io, F, R> {
+pub(crate) struct ServerSelectorImpl<St, C, T, F, R> {
     check: Rc<F>,
     connect: Rc<C>,
     handler: Rc<T>,
@@ -645,27 +518,25 @@ pub(crate) struct ServerSelectorImpl<St, C, T, Io, F, R> {
     max_qos: Option<QoS>,
     disconnect_timeout: Seconds,
     max_topic_alias: u16,
-    time: Timer,
-    _t: marker::PhantomData<(St, Io, R)>,
+    _t: marker::PhantomData<(St, R)>,
 }
 
-impl<St, C, T, Io, F, R> Service for ServerSelectorImpl<St, C, T, Io, F, R>
+impl<St, C, T, F, R> Service<SelectItem> for ServerSelectorImpl<St, C, T, F, R>
 where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
-    F: Fn(&Handshake<Io>) -> R + 'static,
+    St: 'static,
+    F: Fn(&Handshake) -> R + 'static,
     R: Future<Output = Result<bool, C::Error>>,
-    C: Service<Request = Handshake<Io>, Response = HandshakeAck<Io, St>> + 'static,
+    C: Service<Handshake, Response = HandshakeAck<St>> + 'static,
     C::Error: fmt::Debug,
     T: ServiceFactory<
-            Config = Session<St>,
-            Request = DispatchItem<Rc<MqttShared>>,
+            DispatchItem<Rc<MqttShared>>,
+            Session<St>,
             Response = Option<mqtt::Packet>,
             Error = MqttError<C::Error>,
             InitError = MqttError<C::Error>,
         > + 'static,
 {
-    type Request = SelectItem<Io>;
-    type Response = Either<SelectItem<Io>, ()>;
+    type Response = Either<SelectItem, ()>;
     type Error = MqttError<C::Error>;
     type Future = Pin<Box<dyn Future<Output = Result<Self::Response, Self::Error>>>>;
 
@@ -680,21 +551,20 @@ where
     }
 
     #[inline]
-    fn call(&self, req: Self::Request) -> Self::Future {
+    fn call(&self, req: SelectItem) -> Self::Future {
         log::trace!("Start connection handshake");
 
         let check = self.check.clone();
         let connect = self.connect.clone();
         let handler = self.handler.clone();
         let timeout = self.disconnect_timeout;
-        let time = self.time.clone();
         let max_qos = self.max_qos;
         let max_size = self.max_size;
         let mut max_receive = self.max_receive;
         let mut max_topic_alias = self.max_topic_alias;
 
         Box::pin(async move {
-            let (mut hnd, state, mut delay) = req;
+            let (mut hnd, mut delay) = req;
 
             let result = if let Some(ref mut delay) = delay {
                 let fut = (&*check)(&hnd);
@@ -707,7 +577,7 @@ where
             };
 
             if !result.map_err(MqttError::Service)? {
-                Ok(Either::Left((hnd, state, delay)))
+                Ok(Either::Left((hnd, delay)))
             } else {
                 // set max outbound (encoder) packet size
                 if let Some(size) = hnd.packet().max_packet_size {
@@ -764,12 +634,8 @@ where
                             ack.packet.server_keepalive_sec = Some(ack.keepalive as u16);
                         }
 
-                        state
-                            .send(
-                                &mut ack.io,
-                                &shared.codec,
-                                mqtt::Packet::ConnectAck(Box::new(ack.packet)),
-                            )
+                        ack.io
+                            .send(mqtt::Packet::ConnectAck(Box::new(ack.packet)), &shared.codec)
                             .await?;
 
                         let session = Session::new_v5(
@@ -781,7 +647,7 @@ where
                         let handler = handler.new_service(session).await?;
                         log::trace!("Connection handler is created, starting dispatcher");
 
-                        Dispatcher::with(ack.io, shared.state.clone(), shared, handler, time)
+                        Dispatcher::new(ack.io, shared, handler)
                             .keepalive_timeout(Seconds(ack.keepalive))
                             .disconnect_timeout(timeout)
                             .await?;
@@ -790,24 +656,14 @@ where
                     None => {
                         log::trace!("Failed to complete handshake: {:#?}", ack.packet);
 
-                        if ack.shared.state.is_open()
-                            && ack
-                                .shared
-                                .state
-                                .write()
-                                .encode(
-                                    mqtt::Packet::ConnectAck(Box::new(ack.packet)),
-                                    &ack.shared.codec,
-                                )
-                                .is_ok()
-                        {
-                            WriteTask::shutdown(
-                                Rc::new(RefCell::new(ack.io)),
-                                ack.shared.state.clone(),
+                        ack.io
+                            .send(
+                                mqtt::Packet::ConnectAck(Box::new(ack.packet)),
+                                &ack.shared.codec,
                             )
-                            .await;
-                        }
-                        Err(MqttError::Disconnected)
+                            .await?;
+                        let _ = ack.io.shutdown().await;
+                        Err(MqttError::Disconnected(None))
                     }
                 }
             }
